"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@typescript-eslint/utils");
const create_rule_1 = require("../internal/create-rule");
const RandExp = require("randexp");
module.exports = (0, create_rule_1.createRule)({
    name: 'message-pattern',
    meta: {
        type: 'suggestion',
        docs: {
            description: 'Checks if always(), never(), and assert() messages match a pattern.',
            // category: 'Best Practices',
            // recommended: 'error'
        },
        messages: {
            missing: "missing",
            mismatch: "mismatch"
        },
        schema: [{
                type: 'array',
                items: {
                    type: 'string'
                }
            }],
        fixable: "code"
    },
    defaultOptions: [[]],
    create: (context, [options]) => {
        const ASSERTION_EXPRESSION_NAMES = ['assert', 'always', 'never'];
        const DEFAULT_REGEX = /^.*$/;
        return {
            CallExpression: (node) => {
                var _a;
                if (ASSERTION_EXPRESSION_NAMES.includes(node.callee.name)) {
                    const patterns = options !== null && options !== void 0 ? options : [];
                    const args = [...node.arguments];
                    const firstArgument = args.shift();
                    if (args.length < patterns.length) {
                        const lastArgument = (_a = args[args.length - 1]) !== null && _a !== void 0 ? _a : firstArgument;
                        const numberOfMissingArgs = patterns.length - args.length;
                        const missingPatterns = patterns.slice(patterns.length - numberOfMissingArgs);
                        context.report({
                            loc: {
                                start: {
                                    line: lastArgument.loc.end.line,
                                    column: lastArgument.loc.end.column
                                },
                                end: {
                                    line: lastArgument.loc.end.line,
                                    column: lastArgument.loc.end.column
                                }
                            },
                            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                            // @ts-ignore
                            message: getArgumentCountFailureString(patterns.length, args.length),
                            fix: fixer => fixer.insertTextAfter(lastArgument, getFixArgumentCountFailure(missingPatterns))
                        });
                    }
                    args.forEach((arg, index) => {
                        var _a;
                        const option = patterns[index];
                        if (option === false) {
                            return;
                        }
                        const pattern = (_a = new RegExp(option)) !== null && _a !== void 0 ? _a : DEFAULT_REGEX;
                        if (!(arg.type === utils_1.AST_NODE_TYPES.Literal || arg.type === utils_1.AST_NODE_TYPES.TemplateLiteral)) {
                            return;
                        }
                        const stringValue = context.getSourceCode().getText(arg).slice(1, -1);
                        if (pattern.test(stringValue)) {
                            return;
                        }
                        context.report({
                            node: arg,
                            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                            // @ts-ignore
                            message: getPatternFailureString(pattern, index + 1),
                            fix: fixer => fixer.replaceText(arg, getFixPatternFailure(arg, pattern))
                        });
                    });
                }
            }
        };
        function getPatternFailureString(pattern, index) {
            return `Assertion message ${index} has to match ${pattern}.`;
        }
        function getArgumentCountFailureString(expected, provided) {
            if (provided === 0) {
                return `Expected ${expected} assertion messages but none were provided.`;
            }
            if (provided === 1) {
                return `Expected ${expected} assertion messages but only ${provided} was provided.`;
            }
            return `Expected ${expected} assertion messages but only ${provided} were provided.`;
        }
        function getFixPatternFailure(arg, pattern) {
            const randExp = new RandExp(pattern);
            const fix = randExp.gen();
            const text = context.getSourceCode().getText(arg);
            const quoteType = text[0];
            return `${quoteType}${fix}${quoteType}`;
        }
        function getFixArgumentCountFailure(patterns) {
            const fixes = patterns.map((pattern) => {
                if (pattern === false) {
                    return `""`;
                }
                const randExp = new RandExp(pattern);
                return `"${randExp.gen()}"`;
            });
            return ', ' + fixes.join(', ');
        }
    }
});
//# sourceMappingURL=message-pattern.js.map